@@ -25,13 +25,14 @@
 ;;; Code:
 
 (require 'ht)
-(require 'dag-draw)
 (require 'dag-draw-core)
 
-;; Forward declarations for functions from modules that require us (avoid circular deps)
+;; Forward declarations for existing modules
 (declare-function dag-draw--count-total-crossings "dag-draw-pass2-ordering")
 (declare-function dag-draw--organize-by-ranks "dag-draw-pass2-ordering")
 (declare-function dag-draw-balance-ranks "dag-draw-rank-balancing")
+(declare-function dag-draw--debug-spacing-calculation "dag-draw-quality")
+(declare-function dag-draw--calculate-max-required-rank-separation "dag-draw-quality")
 
 ;;; A1: Expose hierarchical structure (Section 1.1, line 43)
 
@@ -52,9 +53,8 @@ higher ranks."
       ;; Count edges that follow hierarchical flow (rank increase)
       (progn
         (dolist (edge (dag-draw-graph-edges graph))
-          (let* ((nodes (dag-draw--edge-nodes graph edge))
-                 (from-node (car nodes))
-                 (to-node (cdr nodes)))
+          (let ((from-node (dag-draw-get-node graph (dag-draw-edge-from-node edge)))
+                (to-node (dag-draw-get-node graph (dag-draw-edge-to-node edge))))
             (when (and from-node to-node
                        (dag-draw-node-rank from-node)
                        (dag-draw-node-rank to-node)
@@ -81,9 +81,8 @@ means all edges flow in the primary direction (top-down)."
       ;; Determine primary direction (top-down in this case)
       (progn
         (dolist (edge edges)
-          (let* ((nodes (dag-draw--edge-nodes graph edge))
-                 (from-node (car nodes))
-                 (to-node (cdr nodes)))
+          (let ((from-node (dag-draw-get-node graph (dag-draw-edge-from-node edge)))
+                (to-node (dag-draw-get-node graph (dag-draw-edge-to-node edge))))
             (when (and from-node to-node
                        (dag-draw-node-rank from-node)
                        (dag-draw-node-rank to-node))
@@ -161,9 +160,8 @@ Returns a hash table with bend metrics:
 
     ;; For each edge, analyze bending characteristics
     (dolist (edge (dag-draw-graph-edges graph))
-      (let* ((nodes (dag-draw--edge-nodes graph edge))
-             (from-node (car nodes))
-             (to-node (cdr nodes))
+      (let* ((from-node (dag-draw-get-node graph (dag-draw-edge-from-node edge)))
+             (to-node (dag-draw-get-node graph (dag-draw-edge-to-node edge)))
              (bend-angle (dag-draw--calculate-edge-bend graph from-node to-node)))
 
         (when bend-angle
@@ -214,9 +212,8 @@ Returns a hash table with length metrics:
         (lengths '()))
 
     (dolist (edge (dag-draw-graph-edges graph))
-      (let* ((nodes (dag-draw--edge-nodes graph edge))
-             (from-node (car nodes))
-             (to-node (cdr nodes))
+      (let* ((from-node (dag-draw-get-node graph (dag-draw-edge-from-node edge)))
+             (to-node (dag-draw-get-node graph (dag-draw-edge-to-node edge)))
              (length (dag-draw--calculate-edge-length from-node to-node)))
 
         (when length
@@ -259,9 +256,8 @@ all connected nodes are within the proximity threshold (100 units)."
 
     ;; For each pair of connected nodes, check if they're close
     (dolist (edge (dag-draw-graph-edges graph))
-      (let* ((nodes (dag-draw--edge-nodes graph edge))
-             (from-node (car nodes))
-             (to-node (cdr nodes))
+      (let* ((from-node (dag-draw-get-node graph (dag-draw-edge-from-node edge)))
+             (to-node (dag-draw-get-node graph (dag-draw-edge-to-node edge)))
              (edge-distance (dag-draw--calculate-edge-length from-node to-node)))
 
         (when edge-distance
